# $Id: pcap.py 293 2006-01-22 03:22:07Z dugsong $

"""Libpcap file format."""

import sys, time
import dpkt

TCPDUMP_MAGIC = 0xa1b2c3d4L
PCAP_VERSION_MAJOR = 2
PCAP_VERSION_MINOR = 4

DLT_NULL =      0
DLT_EN10MB =    1
DLT_EN3MB =     2
DLT_AX25 =      3
DLT_PRONET =    4
DLT_CHAOS =     5
DLT_IEEE802 =   6
DLT_ARCNET =    7
DLT_SLIP =      8
DLT_PPP =       9
DLT_FDDI =      10
# XXX - Linux
DLT_LINUX_SLL = 113
# XXX - OpenBSD
DLT_PFLOG =     117
DLT_PFSYNC =    18
if sys.platform.find('openbsd') != -1:
    DLT_LOOP =          12
    DLT_RAW =           14
else:
    DLT_LOOP =          108
    DLT_RAW =           12

dltoff = { DLT_NULL:4, DLT_EN10MB:14, DLT_IEEE802:22, DLT_ARCNET:6,
           DLT_SLIP:16, DLT_PPP:4, DLT_FDDI:21, DLT_PFLOG:48, DLT_PFSYNC:4,
           DLT_LOOP:4, DLT_RAW:0, DLT_LINUX_SLL:16 }

class PcapFileHdr(dpkt.Packet):
    """pcap file header."""
    __hdr__ = (
        ('magic', 'I', TCPDUMP_MAGIC),
        ('v_major', 'H', PCAP_VERSION_MAJOR),
        ('v_minor', 'H', PCAP_VERSION_MINOR),
        ('thiszone', 'I', 0),
        ('sigfigs', 'I', 0),
        ('snaplen', 'I', 1500),
        ('linktype', 'I', 1),
        )
    __byte_order__ = '@'

class PcapPktHdr(dpkt.Packet):
    """pcap packet header."""
    __hdr__ = (
        ('tv_sec', 'I', 0),
        ('tv_usec', 'I', 0),
        ('caplen', 'I', 0),
        ('len', 'I', 0),
        )
    __byte_order__ = '@'

class PcapDumper(object):
    """Simple pcap dumpfile writer."""
    def __init__(self, filename, snaplen=1500, linktype=DLT_EN10MB):
        self.__f = open(filename, 'w')
        fh = PcapFileHdr(snaplen=snaplen, linktype=linktype)
        self.__f.write(str(fh))

    def append(self, pkt, ts=None):
        if ts is None:
            ts = time.time()
        s = str(pkt)
        n = len(s)
        ph = PcapPktHdr(tv_sec=int(ts),
                        tv_usec=int((int(ts) - float(ts)) * 1000000.0),
                        caplen=n, len=n)
        self.__f.write(str(ph))
        self.__f.write(s)

    def close(self):
        self.__f.close()

class PcapReader(object):
    """Simple pypcap-compatible pcap file reader."""
    
    def __init__(self, name):
        self.name = name
        self.__f = open(self.name)
        self.__fh = PcapFileHdr(self.__f.read(PcapFileHdr.__hdr_len__))
        self.snaplen = self.__fh.snaplen
        self.dloff = dltoff[self.__fh.linktype]
        self.filter = ''
        self.fd = self.__f.fileno()

    def fileno(self):
        return self.fd
    
    def datalink(self):
        return self.__fh.linktype
    
    def setfilter(self, value, optimize=1):
        return NotImplementedError

    def readpkts(self):
        return list(self)
    
    def dispatch(self, cnt, callback, *args):
        if cnt > 0:
            for i in range(cnt):
                ts, pkt = self.next()
                callback(ts, pkt, *args)
        else:
            for ts, pkt in self:
                callback(ts, pkt, *args)

    def loop(self, callback, *args):
        self.dispatch(0, callback, *args)
    
    def __iter__(self):
        self.__f.seek(PcapFileHdr.__hdr_len__)
        while 1:
            buf = self.__f.read(PcapPktHdr.__hdr_len__)
            if not buf: break
            hdr = PcapPktHdr(buf)
            buf = self.__f.read(hdr.caplen)
            yield (hdr.tv_sec + (hdr.tv_usec / 1000000.0), buf)
